@@ -1,6 +1,6 @@
 
 /*  A Bison parser, made from ./getdate.y
- with Bison version Bison version A2.3 (Andrew Consortium)
+ by  Bison version A2.5 (Andrew Consortium)
   */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -114,6 +114,14 @@ struct timeb {
 #define bcopy(from, to, len) memcpy ((to), (from), (len))
 #endif
 
+#if defined (STDC_HEADERS)
+#include <stdlib.h>
+#endif
+
+#if defined (HAVE_ALLOCA_H)
+#include <alloca.h>
+#endif
+
 extern struct tm	*gmtime();
 extern struct tm	*localtime();
 
@@ -185,7 +193,7 @@ static time_t	yyRelMonth;
 static time_t	yyRelSeconds;
 
 
-#line 167 "./getdate.y"
+#line 175 "./getdate.y"
 typedef union {
     time_t		Number;
     enum _MERIDIAN	Meridian;
@@ -265,18 +273,18 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   181,   182,   185,   188,   191,   194,   197,   200,   203,   209,
-   215,   222,   228,   238,   242,   246,   253,   257,   261,   267,
-   271,   276,   282,   288,   292,   297,   301,   308,   312,   315,
-   318,   321,   324,   327,   330,   333,   336,   339,   344,   371,
-   374
+   189,   190,   193,   196,   199,   202,   205,   208,   211,   217,
+   223,   230,   236,   246,   250,   254,   261,   265,   269,   275,
+   279,   284,   290,   296,   300,   305,   309,   316,   320,   323,
+   326,   329,   332,   335,   338,   341,   344,   347,   352,   379,
+   382
 };
 #endif
 
 
 #if YYDEBUG != 0
 
-static const char * const yytname[] = {   "$","error","$illegal.","tAGO","tDAY",
+static const char * const yytname[] = {   "$","error","$undefined.","tAGO","tDAY",
 "tDAYZONE","tID","tMERIDIAN","tMINUTE_UNIT","tMONTH","tMONTH_UNIT","tSEC_UNIT",
 "tSNUMBER","tUNUMBER","tZONE","tDST","':'","','","'/'","spec","item","time",
 "zone","day","date","rel","relunit","number","o_merid", NULL
@@ -345,14 +353,14 @@ static const short yycheck[] = {     0,
     13,    47
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/usr/lib/bison.simple"
+#line 3 "/usr/share/bison.simple"
 
 /* Skeleton output parser for bison,
-   Copyright (C) 1984, 1989, 1990 Bob Corbett and Richard Stallman
+   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 1, or (at your option)
+   the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -364,6 +372,10 @@ static const short yycheck[] = {     0,
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/* As a special exception, when this file is copied by Bison into a
+   Bison output file, you may use that output file without restriction.
+   This special exception was added by the Free Software Foundation
+   in version 1.24 of Bison.  */
 
 #ifndef alloca
 #ifdef __GNUC__
@@ -437,10 +449,18 @@ while (0)
 
 #ifdef YYPURE
 #ifdef YYLSP_NEEDED
+#ifdef YYLEX_PARAM
+#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
+#else
 #define YYLEX		yylex(&yylval, &yylloc)
+#endif
+#else /* not YYLSP_NEEDED */
+#ifdef YYLEX_PARAM
+#define YYLEX		yylex(&yylval, YYLEX_PARAM)
 #else
 #define YYLEX		yylex(&yylval)
 #endif
+#endif /* not YYLSP_NEEDED */
 #endif
 
 /* If nonreentrant, generate the variables here */
@@ -488,14 +508,14 @@ int yyparse (void);
 #endif
 
 #if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
-#define __yy_bcopy(FROM,TO,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
+#define __yy_memcpy(FROM,TO,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
 #else				/* not GNU C or C++ */
 #ifndef __cplusplus
 
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_bcopy (from, to, count)
+__yy_memcpy (from, to, count)
      char *from;
      char *to;
      int count;
@@ -513,7 +533,7 @@ __yy_bcopy (from, to, count)
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_bcopy (char *from, char *to, int count)
+__yy_memcpy (char *from, char *to, int count)
 {
   register char *f = from;
   register char *t = to;
@@ -526,9 +546,24 @@ __yy_bcopy (char *from, char *to, int count)
 #endif
 #endif
 
-#line 184 "/usr/lib/bison.simple"
+#line 192 "/usr/share/bison.simple"
+
+/* The user can define YYPARSE_PARAM as the name of an argument to be passed
+   into yyparse.  The argument should have type void *.
+   It should actually point to an object.
+   Grammar actions can access the variable by casting it
+   to the proper pointer type.  */
+
+#ifdef YYPARSE_PARAM
+#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
+#else
+#define YYPARSE_PARAM
+#define YYPARSE_PARAM_DECL
+#endif
+
 int
-yyparse()
+yyparse(YYPARSE_PARAM)
+     YYPARSE_PARAM_DECL
 {
   register int yystate;
   register int yyn;
@@ -644,12 +679,12 @@ yynewstate:
       if (yystacksize > YYMAXDEPTH)
 	yystacksize = YYMAXDEPTH;
       yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
-      __yy_bcopy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
+      __yy_memcpy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
       yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
-      __yy_bcopy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
+      __yy_memcpy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
 #ifdef YYLSP_NEEDED
       yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
-      __yy_bcopy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
+      __yy_memcpy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
 #endif
 #endif /* no yyoverflow */
 
@@ -810,37 +845,37 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 185 "./getdate.y"
+#line 193 "./getdate.y"
 {
 	    yyHaveTime++;
 	;
     break;}
 case 4:
-#line 188 "./getdate.y"
+#line 196 "./getdate.y"
 {
 	    yyHaveZone++;
 	;
     break;}
 case 5:
-#line 191 "./getdate.y"
+#line 199 "./getdate.y"
 {
 	    yyHaveDate++;
 	;
     break;}
 case 6:
-#line 194 "./getdate.y"
+#line 202 "./getdate.y"
 {
 	    yyHaveDay++;
 	;
     break;}
 case 7:
-#line 197 "./getdate.y"
+#line 205 "./getdate.y"
 {
 	    yyHaveRel++;
 	;
     break;}
 case 9:
-#line 203 "./getdate.y"
+#line 211 "./getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -849,7 +884,7 @@ case 9:
 	;
     break;}
 case 10:
-#line 209 "./getdate.y"
+#line 217 "./getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -858,7 +893,7 @@ case 10:
 	;
     break;}
 case 11:
-#line 215 "./getdate.y"
+#line 223 "./getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -868,7 +903,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 222 "./getdate.y"
+#line 230 "./getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -877,7 +912,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 228 "./getdate.y"
+#line 236 "./getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -888,56 +923,56 @@ case 13:
 	;
     break;}
 case 14:
-#line 238 "./getdate.y"
+#line 246 "./getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSToff;
 	;
     break;}
 case 15:
-#line 242 "./getdate.y"
+#line 250 "./getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSTon;
 	;
     break;}
 case 16:
-#line 247 "./getdate.y"
+#line 255 "./getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number;
 	    yyDSTmode = DSTon;
 	;
     break;}
 case 17:
-#line 253 "./getdate.y"
+#line 261 "./getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 18:
-#line 257 "./getdate.y"
+#line 265 "./getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	;
     break;}
 case 19:
-#line 261 "./getdate.y"
+#line 269 "./getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 20:
-#line 267 "./getdate.y"
+#line 275 "./getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 21:
-#line 271 "./getdate.y"
+#line 279 "./getdate.y"
 {
 	    yyMonth = yyvsp[-4].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -945,7 +980,7 @@ case 21:
 	;
     break;}
 case 22:
-#line 276 "./getdate.y"
+#line 284 "./getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -954,7 +989,7 @@ case 22:
 	;
     break;}
 case 23:
-#line 282 "./getdate.y"
+#line 290 "./getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -963,14 +998,14 @@ case 23:
 	;
     break;}
 case 24:
-#line 288 "./getdate.y"
+#line 296 "./getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 25:
-#line 292 "./getdate.y"
+#line 300 "./getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -978,14 +1013,14 @@ case 25:
 	;
     break;}
 case 26:
-#line 297 "./getdate.y"
+#line 305 "./getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	;
     break;}
 case 27:
-#line 301 "./getdate.y"
+#line 309 "./getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -993,68 +1028,68 @@ case 27:
 	;
     break;}
 case 28:
-#line 308 "./getdate.y"
+#line 316 "./getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMonth = -yyRelMonth;
 	;
     break;}
 case 30:
-#line 315 "./getdate.y"
+#line 323 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	;
     break;}
 case 31:
-#line 318 "./getdate.y"
+#line 326 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	;
     break;}
 case 32:
-#line 321 "./getdate.y"
+#line 329 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[0].Number * 60L;
 	;
     break;}
 case 33:
-#line 324 "./getdate.y"
+#line 332 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	;
     break;}
 case 34:
-#line 327 "./getdate.y"
+#line 335 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	;
     break;}
 case 35:
-#line 330 "./getdate.y"
+#line 338 "./getdate.y"
 {
 	    yyRelSeconds++;
 	;
     break;}
 case 36:
-#line 333 "./getdate.y"
+#line 341 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 37:
-#line 336 "./getdate.y"
+#line 344 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 38:
-#line 339 "./getdate.y"
+#line 347 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[0].Number;
 	;
     break;}
 case 39:
-#line 344 "./getdate.y"
+#line 352 "./getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 		yyYear = yyvsp[0].Number;
@@ -1082,20 +1117,20 @@ case 39:
 	;
     break;}
 case 40:
-#line 371 "./getdate.y"
+#line 379 "./getdate.y"
 {
 	    yyval.Meridian = MER24;
 	;
     break;}
 case 41:
-#line 374 "./getdate.y"
+#line 382 "./getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 465 "/usr/lib/bison.simple"
+#line 487 "/usr/share/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1291,7 +1326,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 379 "./getdate.y"
+#line 387 "./getdate.y"
 
 
 /* Month and day table. */
@@ -1810,25 +1845,30 @@ get_date(p, now)
     struct timeb	ftz;
     time_t		Start;
     time_t		tod;
+    time_t nowtime;
 
     yyInput = p;
     if (now == NULL) {
         now = &ftz;
-	(void)time(&ftz.time);
+	(void)time (&nowtime);
 
-	if (! (tm = gmtime (&ftz.time)))
+	if (! (tm = gmtime (&nowtime)))
 	    return -1;
 	gmt = *tm;	/* Make a copy, in case localtime modifies *tm.  */
 
-	if (! (tm = localtime (&ftz.time)))
+	if (! (tm = localtime (&nowtime)))
 	    return -1;
 	
 	ftz.timezone = difftm (&gmt, tm) / 60;
 	if(tm->tm_isdst)
 	    ftz.timezone += 60;
     }
+    else
+    {
+	nowtime = now->time;
+    }
 
-    tm = localtime(&now->time);
+    tm = localtime(&nowtime);
     yyYear = tm->tm_year;
     yyMonth = tm->tm_mon + 1;
     yyDay = tm->tm_mday;
@@ -1857,7 +1897,7 @@ get_date(p, now)
 	    return -1;
     }
     else {
-	Start = now->time;
+	Start = nowtime;
 	if (!yyHaveRel)
 	    Start -= ((tm->tm_hour * 60L + tm->tm_min) * 60L) + tm->tm_sec;
     }
