@@ -7,11 +7,7 @@
  */
 
 #include "cvs.h"
-
-#ifndef lint
-static const char rcsid[] = "$CVSid: @(#)logmsg.c 1.48 94/09/29 $";
-USE(rcsid)
-#endif
+#include "getline.h"
 
 static int find_type PROTO((Node * p, void *closure));
 static int fmt_proc PROTO((Node * p, void *closure));
@@ -25,7 +21,7 @@ static void setup_tmpfile PROTO((FILE * xfp, char *xprefix, List * changes));
 static int editinfo_proc PROTO((char *repository, char *template));
 
 static FILE *fp;
-static char *strlist;
+static char *str_list;
 static char *editinfo_editor;
 static Ctype type;
 
@@ -116,10 +112,8 @@ fmt_proc (p, closure)
  * editor on the file.  The header garbage in the resultant file is then
  * stripped and the log message is stored in the "message" argument.
  * 
- * rcsinfo - is the name of a file containing lines tacked onto the end of the
- * RCS info offered to the user for editing. If specified, the '-m' flag to
- * "commit" is disabled -- users are forced to run the editor.
- * 
+ * If REPOSITORY is non-NULL, process rcsinfo for that repository; if it
+ * is NULL, use the CVSADM_TEMPLATE file instead.
  */
 void
 do_editor (dir, messagep, repository, changes)
@@ -129,7 +123,10 @@ do_editor (dir, messagep, repository, changes)
     List *changes;
 {
     static int reuse_log_message = 0;
-    char line[MAXLINELEN], fname[L_tmpnam+1];
+    char *line;
+    int line_length;
+    size_t line_chars_allocated;
+    char fname[L_tmpnam+1];
     struct stat pre_stbuf, post_stbuf;
     int retcode = 0;
     char *p;
@@ -137,6 +134,10 @@ do_editor (dir, messagep, repository, changes)
     if (noexec || reuse_log_message)
 	return;
 
+    /* Abort creation of temp file if no editor is defined */
+    if (strcmp (Editor, "") == 0 && !editinfo_editor)
+	error(1, 0, "no editor defined, must use -e or -m");
+
     /* Create a temporary file */
     (void) tmpnam (fname);
   again:
@@ -156,6 +157,41 @@ do_editor (dir, messagep, repository, changes)
     if (repository != NULL)
 	/* tack templates on if necessary */
 	(void) Parse_Info (CVSROOTADM_RCSINFO, repository, rcsinfo_proc, 1);
+    else
+    {
+	FILE *tfp;
+	char buf[1024];
+	char *p;
+	size_t n;
+	size_t nwrite;
+
+	/* Why "b"?  */
+	tfp = fopen (CVSADM_TEMPLATE, "rb");
+	if (tfp == NULL)
+	{
+	    if (!existence_error (errno))
+		error (1, errno, "cannot read %s", CVSADM_TEMPLATE);
+	}
+	else
+	{
+	    while (!feof (tfp))
+	    {
+		n = fread (buf, 1, sizeof buf, tfp);
+		nwrite = n;
+		p = buf;
+		while (nwrite > 0)
+		{
+		    n = fwrite (p, 1, nwrite, fp);
+		    nwrite -= n;
+		    p += n;
+		}
+		if (ferror (tfp))
+		    error (1, errno, "cannot read %s", CVSADM_TEMPLATE);
+	    }
+	    if (fclose (tfp) < 0)
+		error (0, errno, "cannot close %s", CVSADM_TEMPLATE);
+	}
+    }
 
     (void) fprintf (fp,
   "%s----------------------------------------------------------------------\n",
@@ -163,7 +199,7 @@ do_editor (dir, messagep, repository, changes)
     (void) fprintf (fp,
   "%sEnter Log.  Lines beginning with `%s' are removed automatically\n%s\n",
 		    CVSEDITPREFIX, CVSEDITPREFIX, CVSEDITPREFIX);
-    if (dir != NULL)
+    if (dir != NULL && *dir)
 	(void) fprintf (fp, "%sCommitting in %s\n%s\n", CVSEDITPREFIX,
 			dir, CVSEDITPREFIX);
     if (changes != NULL)
@@ -173,7 +209,8 @@ do_editor (dir, messagep, repository, changes)
 		    CVSEDITPREFIX);
 
     /* finish off the temp file */
-    (void) fclose (fp);
+    if (fclose (fp) == EOF)
+        error (1, errno, "%s", fname);
     if (stat (fname, &pre_stbuf) == -1)
 	pre_stbuf.st_mtime = 0;
 
@@ -206,25 +243,36 @@ do_editor (dir, messagep, repository, changes)
 	*messagep = NULL;
     else
     {
+	/* On NT, we might read less than st_size bytes, but we won't
+	   read more.  So this works.  */
 	*messagep = (char *) xmalloc (post_stbuf.st_size + 1);
  	*messagep[0] = '\0';
     }
 
-/* !!! XXX FIXME: fgets is broken.  This should not have any line
-   length limits. */
+    line = NULL;
+    line_chars_allocated = 0;
 
     if (*messagep)
     {
 	p = *messagep;
-	while (fgets (line, sizeof (line), fp) != NULL)
+	while (1)
 	{
+	    line_length = getline (&line, &line_chars_allocated, fp);
+	    if (line_length == -1)
+	    {
+		if (ferror (fp))
+		    error (0, errno, "warning: cannot read %s", fname);
+		break;
+	    }
 	    if (strncmp (line, CVSEDITPREFIX, sizeof (CVSEDITPREFIX) - 1) == 0)
 		continue;
 	    (void) strcpy (p, line);
-	    p += strlen (line);
+	    p += line_length;
 	}
     }
-    (void) fclose (fp);
+    if (fclose (fp) < 0)
+	error (0, errno, "warning: cannot close %s", fname);
+
     if (pre_stbuf.st_mtime == post_stbuf.st_mtime ||
 	*messagep == NULL ||
 	strcmp (*messagep, "\n") == 0)
@@ -235,9 +283,9 @@ do_editor (dir, messagep, repository, changes)
 	    (void) printf ("a)bort, c)ontinue, e)dit, !)reuse this message unchanged for remaining dirs\n");
 	    (void) printf ("Action: (continue) ");
 	    (void) fflush (stdout);
-	    *line = '\0';
-	    (void) fgets (line, sizeof (line), stdin);
-	    if (*line == '\0' || *line == '\n' || *line == 'c' || *line == 'C')
+	    line_length = getline (&line, &line_chars_allocated, stdin);
+	    if (line_length <= 0
+		    || *line == '\n' || *line == 'c' || *line == 'C')
 		break;
 	    if (*line == 'a' || *line == 'A')
 		error (1, 0, "aborted by user");
@@ -251,7 +299,10 @@ do_editor (dir, messagep, repository, changes)
 	    (void) printf ("Unknown input\n");
 	}
     }
-    (void) unlink_file (fname);
+    if (line)
+	free (line);
+    if (unlink_file (fname) < 0)
+	error (0, errno, "warning: cannot remove temp file %s", fname);
 }
 
 /*
@@ -267,7 +318,6 @@ rcsinfo_proc (repository, template)
 {
     static char *last_template;
     FILE *tfp;
-    char line[MAXLINELEN];
 
     /* nothing to do if the last one included is the same as this one */
     if (last_template && strcmp (last_template, template) == 0)
@@ -278,14 +328,22 @@ rcsinfo_proc (repository, template)
 
     if ((tfp = fopen (template, "r")) != NULL)
     {
-	while (fgets (line, sizeof (line), tfp) != NULL)
+	char *line = NULL;
+	size_t line_chars_allocated = 0;
+
+	while (getline (&line, &line_chars_allocated, tfp) >= 0)
 	    (void) fputs (line, fp);
-	(void) fclose (tfp);
+	if (ferror (tfp))
+	    error (0, errno, "warning: cannot read %s", template);
+	if (fclose (tfp) < 0)
+	    error (0, errno, "warning: cannot close %s", template);
+	if (line)
+	    free (line);
 	return (0);
     }
     else
     {
-	error (0, 0, "Couldn't open rcsinfo template file %s", template);
+	error (0, errno, "Couldn't open rcsinfo template file %s", template);
 	return (1);
     }
 }
@@ -326,9 +384,9 @@ Update_Logfile (repository, xmessage, xrevision, xlogfp, xchanges)
     srepos = Short_Repository (repository);
 
     /* allocate a chunk of memory to hold the title string */
-    if (!strlist)
-	strlist = xmalloc (MAXLISTLEN);
-    strlist[0] = '\0';
+    if (!str_list)
+	str_list = xmalloc (MAXLISTLEN);
+    str_list[0] = '\0';
 
     type = T_TITLE;
     (void) walklist (changes, title_proc, NULL);
@@ -338,12 +396,12 @@ Update_Logfile (repository, xmessage, xrevision, xlogfp, xchanges)
     (void) walklist (changes, title_proc, NULL);
     type = T_REMOVED;
     (void) walklist (changes, title_proc, NULL);
-    title = xmalloc (strlen (srepos) + strlen (strlist) + 1 + 2); /* for 's */
-    (void) sprintf (title, "'%s%s'", srepos, strlist);
+    title = xmalloc (strlen (srepos) + strlen (str_list) + 1 + 2); /* for 's */
+    (void) sprintf (title, "'%s%s'", srepos, str_list);
 
     /* to be nice, free up this chunk of memory */
-    free (strlist);
-    strlist = (char *) NULL;
+    free (str_list);
+    str_list = (char *) NULL;
 
     /* call Parse_Info to do the actual logfile updates */
     (void) Parse_Info (CVSROOTADM_LOGINFO, repository, update_logfile_proc, 1);
@@ -365,7 +423,7 @@ update_logfile_proc (repository, filter)
 }
 
 /*
- * concatenate each name onto strlist
+ * concatenate each name onto str_list
  */
 static int
 title_proc (p, closure)
@@ -374,8 +432,8 @@ title_proc (p, closure)
 {
     if (p->data == (char *) type)
     {
-	(void) strcat (strlist, " ");
-	(void) strcat (strlist, p->key);
+	(void) strcat (str_list, " ");
+	(void) strcat (str_list, p->key);
     }
     return (0);
 }
@@ -402,16 +460,17 @@ logfile_write (repository, filter, title, message, revision, logfp, changes)
     List *changes;
 {
     char cwd[PATH_MAX];
-    FILE *pipefp, *Popen ();
+    FILE *pipefp;
     char *prog = xmalloc (MAXPROGLEN);
     char *cp;
     int c;
+    int pipestatus;
 
     /*
-     * A maximum of 6 %s arguments are supported in the filter
+     * Only 1 %s argument is supported in the filter
      */
-    (void) sprintf (prog, filter, title, title, title, title, title, title);
-    if ((pipefp = Popen (prog, "w")) == NULL)
+    (void) sprintf (prog, filter, title);
+    if ((pipefp = run_popen (prog, "w")) == NULL)
     {
 	if (!noexec)
 	    error (0, 0, "cannot write entry to log filter: %s", prog);
@@ -433,7 +492,8 @@ logfile_write (repository, filter, title, message, revision, logfp, changes)
 	    (void) putc ((char) c, pipefp);
     }
     free (prog);
-    return (pclose (pipefp));
+    pipestatus = pclose (pipefp);
+    return ((pipestatus == -1) || (pipestatus == 127)) ? 1 : 0;
 }
 
 /*
