@@ -21,6 +21,7 @@
  *		commit		Checks files into the repository
  *		diff		Runs diffs between revisions
  *		log		Prints "rlog" information for files
+ *		login		Record user, host, repos, password
  *		add		Adds an entry to the repository
  *		remove		Removes an entry from the repository
  *		status		Status info on the revisions
@@ -33,24 +34,28 @@
  */
 
 #include "cvs.h"
-#include "patchlevel.h"
+
+#ifdef HAVE_WINSOCK_H
+#include <winsock.h>
+#else
+extern int gethostname ();
+#endif
 
 #if HAVE_KERBEROS
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <krb.h>
-#include <pwd.h>
 #ifndef HAVE_KRB_GET_ERR_TEXT
 #define krb_get_err_text(status) krb_err_txt[status]
 #endif
 #endif
 
-#ifndef lint
-static const char rcsid[] = "$CVSid: @(#)main.c 1.78 94/10/07 $\n";
-USE(rcsid)
-#endif
-
 char *program_name;
+char *program_path;
+/*
+ * Initialize comamnd_name to "cvs" so that the first call to
+ * read_cvsrc tries to find global cvs options.
+ */
 char *command_name = "";
 
 /*
@@ -62,6 +67,9 @@ char *command_name = "";
 
 char hostname[MAXHOSTNAMELEN];
 
+#ifdef AUTH_CLIENT_SUPPORT
+int use_authenticating_server = FALSE;
+#endif /* AUTH_CLIENT_SUPPORT */
 int use_editor = TRUE;
 int use_cvsrc = TRUE;
 int cvswrite = !CVSREAD_DFLT;
@@ -70,6 +78,7 @@ int quiet = FALSE;
 int trace = FALSE;
 int noexec = FALSE;
 int logoff = FALSE;
+mode_t cvsumask = UMASK_DFLT;
 
 char *CurDir;
 
@@ -79,12 +88,10 @@ char *CurDir;
 char *Rcsbin = RCSBIN_DFLT;
 char *Editor = EDITOR_DFLT;
 char *CVSroot = CVSROOT_DFLT;
-#ifdef CVSADM_ROOT
 /*
  * The path found in CVS/Root must match $CVSROOT and/or 'cvs -d root'
  */
 char *CVSADM_Root = CVSROOT_DFLT;
-#endif /* CVSADM_ROOT */
 
 int add PROTO((int argc, char **argv));
 int admin PROTO((int argc, char **argv));
@@ -94,6 +101,9 @@ int diff PROTO((int argc, char **argv));
 int history PROTO((int argc, char **argv));
 int import PROTO((int argc, char **argv));
 int cvslog PROTO((int argc, char **argv));
+#ifdef AUTH_CLIENT_SUPPORT
+int login PROTO((int argc, char **argv));
+#endif /* AUTH_CLIENT_SUPPORT */
 int patch PROTO((int argc, char **argv));
 int release PROTO((int argc, char **argv));
 int cvsremove PROTO((int argc, char **argv));
@@ -108,34 +118,55 @@ const struct cmd
     char *nick1;		/* alternate name (e.g. "ci") */
     char *nick2;		/* another alternate names (e.g. "ci") */
     int (*func) ();		/* Function takes (argc, argv) arguments. */
+#ifdef CLIENT_SUPPORT
     int (*client_func) ();	/* Function to do it via the protocol.  */
+#endif
 } cmds[] =
 
 {
-    { "add", "ad", "new", add, client_add },
-    { "admin", "adm", "rcs", admin, client_admin },
-    { "checkout", "co", "get", checkout, client_checkout },
-    { "commit", "ci", "com", commit, client_commit },
-    { "diff", "di", "dif", diff, client_diff },
-    { "export", "exp", "ex", checkout, client_export },
-    { "history", "hi", "his", history, client_history },
-    { "import", "im", "imp", import, client_import },
-    { "log", "lo", "rlog", cvslog, client_log },
-    { "rdiff", "patch", "pa", patch, client_rdiff },
-    { "release", "re", "rel", release, client_release },
-    { "remove", "rm", "delete", cvsremove, client_remove },
-    { "status", "st", "stat", status, client_status },
-    { "rtag", "rt", "rfreeze", rtag, client_rtag },
-    { "tag", "ta", "freeze", tag, client_tag },
-    { "update", "up", "upd", update, client_update },
+#ifdef CLIENT_SUPPORT
+#define CMD_ENTRY(n1, n2, n3, f1, f2) { n1, n2, n3, f1, f2 }
+#else
+#define CMD_ENTRY(n1, n2, n3, f1, f2) { n1, n2, n3, f1 }
+#endif
 
+    CMD_ENTRY("add",      "ad",    "new",     add,       client_add),
+    CMD_ENTRY("admin",    "adm",   "rcs",     admin,     client_admin),
+    CMD_ENTRY("annotate", NULL,    NULL,      annotate,  client_annotate),
+    CMD_ENTRY("checkout", "co",    "get",     checkout,  client_checkout),
+    CMD_ENTRY("commit",   "ci",    "com",     commit,    client_commit),
+    CMD_ENTRY("diff",     "di",    "dif",     diff,      client_diff),
+    CMD_ENTRY("edit",     "edit",  "edit",    edit,      client_edit),
+    CMD_ENTRY("editors",  "editors","editors",editors,   client_editors),
+    CMD_ENTRY("export",   "exp",   "ex",      checkout,  client_export),
+    CMD_ENTRY("history",  "hi",    "his",     history,   client_history),
+    CMD_ENTRY("import",   "im",    "imp",     import,    client_import),
+    CMD_ENTRY("init",     NULL,    NULL,      init,      client_init),
+    CMD_ENTRY("log",      "lo",    "rlog",    cvslog,    client_log),
+#ifdef AUTH_CLIENT_SUPPORT
+    CMD_ENTRY("login",    "logon", "lgn",     login,     login),
+#endif /* AUTH_CLIENT_SUPPORT */
+    CMD_ENTRY("rdiff",    "patch", "pa",      patch,     client_rdiff),
+    CMD_ENTRY("release",  "re",    "rel",     release,   client_release),
+    CMD_ENTRY("remove",   "rm",    "delete",  cvsremove, client_remove),
+    CMD_ENTRY("status",   "st",    "stat",    status,    client_status),
+    CMD_ENTRY("rtag",     "rt",    "rfreeze", rtag,      client_rtag),
+    CMD_ENTRY("tag",      "ta",    "freeze",  tag,       client_tag),
+    CMD_ENTRY("unedit",   "unedit","unedit",  unedit,    client_unedit),
+    CMD_ENTRY("update",   "up",    "upd",     update,    client_update),
+    CMD_ENTRY("watch",    "watch", "watch",   watch,     client_watch),
+    CMD_ENTRY("watchers", "watchers","watchers",watchers,client_watchers),
+#ifdef SERVER_SUPPORT
     /*
      * The client_func is also server because we might have picked up a
      * CVSROOT environment variable containing a colon.  The client will send
      * the real root later.
      */
-    { "server", "server", "server", server, server },
-    { NULL, NULL, NULL, NULL, NULL },
+    CMD_ENTRY("server",   "server", "server", server,    server),
+#endif
+    CMD_ENTRY(NULL, NULL, NULL, NULL, NULL),
+
+#undef CMD_ENTRY
 };
 
 static const char *const usg[] =
@@ -155,32 +186,61 @@ static const char *const usg[] =
     "        -e editor    Use 'editor' for editing log information\n",
     "        -d CVS_root  Overrides $CVSROOT as the root of the CVS tree\n",
     "        -f           Do not use the ~/.cvsrc file\n",
+#ifdef CLIENT_SUPPORT
     "        -z #         Use 'gzip -#' for net traffic if possible.\n",
+#endif
+    "        -s VAR=VAL   Set CVS user variable.\n",
     "\n",
-    "    and where 'command' is:\n",
+    "    and where 'command' is: add, admin, etc. (use the --help-commands\n",
+    "    option for a list of commands)\n",
+    NULL,
+};
+
+static const char *const cmd_usage[] =
+{
+    "CVS commands are:\n",
     "        add          Adds a new file/directory to the repository\n",
     "        admin        Administration front end for rcs\n",
+    "        annotate     Show revision where each line was modified\n",
     "        checkout     Checkout sources for editing\n",
     "        commit       Checks files into the repository\n",
     "        diff         Runs diffs between revisions\n",
+    "        edit         Get ready to edit a watched file\n",
+    "        editors      See who is editing a watched file\n",
     "        history      Shows status of files and users\n",
     "        import       Import sources into CVS, using vendor branches\n",
     "        export       Export sources from CVS, similar to checkout\n",
     "        log          Prints out 'rlog' information for files\n",
+#ifdef AUTH_CLIENT_SUPPORT
+    "        login        Prompt for password for authenticating server.\n",
+#endif /* AUTH_CLIENT_SUPPORT */
     "        rdiff        'patch' format diffs between releases\n",
     "        release      Indicate that a Module is no longer in use\n",
     "        remove       Removes an entry from the repository\n",
     "        status       Status info on the revisions\n",
     "        tag          Add a symbolic tag to checked out version of RCS file\n",
+    "        unedit       Undo an edit command\n",
     "        rtag         Add a symbolic tag to the RCS file\n",
     "        update       Brings work tree in sync with repository\n",
+    "        watch        Set watches\n",
+    "        watchers     See who is watching a file\n",
     NULL,
 };
 
 static RETSIGTYPE
 main_cleanup ()
 {
-    exit (1);
+    exit (EXIT_FAILURE);
+}
+
+static void
+error_cleanup PROTO((void))
+{
+    Lock_Cleanup();
+#ifdef SERVER_SUPPORT
+    if (server_active)
+	server_cleanup (0);
+#endif
 }
 
 int
@@ -189,19 +249,37 @@ main (argc, argv)
     char **argv;
 {
     extern char *version_string;
-    char *cp;
+    extern char *config_string;
+    char *cp, *end;
     const struct cmd *cm;
-    int c, help = FALSE, err = 0;
+    int c, err = 0;
+    static int help = FALSE;
+    static int version_flag = FALSE;
+    static int help_commands = FALSE;
     int rcsbin_update_env, cvs_update_env = 0;
-    char tmp[PATH_MAX];
+    static struct option long_options[] =
+      {
+        {"help", 0, &help, TRUE},
+        {"version", 0, &version_flag, TRUE},
+	{"help-commands", 0, &help_commands, TRUE},
+        {0, 0, 0, 0}
+      };
+    /* `getopt_long' stores the option index here, but right now we
+        don't use it. */
+    int option_index = 0;
+
+    error_set_cleanup (error_cleanup);
+
+/* The socket subsystems on NT and OS2 must be initialized before use */
+#ifdef INITIALIZE_SOCKET_SUBSYSTEM
+        INITIALIZE_SOCKET_SUBSYSTEM();
+#endif /* INITIALIZE_SOCKET_SUBSYSTEM */
 
     /*
      * Just save the last component of the path for error messages
      */
-    if ((program_name = strrchr (argv[0], '/')) == NULL)
-	program_name = argv[0];
-    else
-	program_name++;
+    program_path = xstrdup (argv[0]);
+    program_name = last_component (argv[0]);
 
     CurDir = xmalloc (PATH_MAX);
 #ifndef SERVER_SUPPORT
@@ -224,6 +302,8 @@ main (argc, argv)
  	Editor = cp;
     else if ((cp = getenv (EDITOR2_ENV)) != NULL)
 	Editor = cp;
+    else if ((cp = getenv (EDITOR3_ENV)) != NULL)
+	Editor = cp;
     if ((cp = getenv (CVSROOT_ENV)) != NULL)
     {
 	CVSroot = cp;
@@ -231,12 +311,55 @@ main (argc, argv)
     }
     if (getenv (CVSREAD_ENV) != NULL)
 	cvswrite = FALSE;
+    if ((cp = getenv (CVSUMASK_ENV)) != NULL)
+    {
+	/* FIXME: Should be accepting symbolic as well as numeric mask.  */
+	cvsumask = strtol (cp, &end, 8) & 0777;
+	if (*end != '\0')
+	    error (1, errno, "invalid umask value in %s (%s)",
+		CVSUMASK_ENV, cp);
+    }
+
+    /* This has the effect of setting getopt's ordering to REQUIRE_ORDER,
+       which is what we need to distinguish between global options and
+       command options.  FIXME: It would appear to be possible to do this
+       much less kludgily by passing "+" as the first character to the
+       option string we pass to getopt_long.  */
+    optind = 1;
+
+
+    /* We have to parse the options twice because else there is no
+       chance to avoid reading the global options from ".cvsrc".  Set
+       opterr to 0 for avoiding error messages about invalid options.
+       */
+    opterr = 0;
+
+    while ((c = getopt_long
+            (argc, argv, "f", NULL, NULL))
+           != EOF)
+      {
+	if (c == 'f')
+	    use_cvsrc = FALSE;
+      }
+    
+    /*
+     * Scan cvsrc file for global options.
+     */
+    if (use_cvsrc)
+	read_cvsrc (&argc, &argv, "cvs");
 
     optind = 1;
-    while ((c = getopt (argc, argv, "Qqrwtnlvb:e:d:Hfz:")) != -1)
-    {
+    opterr = 1;
+
+    while ((c = getopt_long
+            (argc, argv, "Qqrwtnlvb:e:d:Hfz:s:", long_options, &option_index))
+           != EOF)
+      {
 	switch (c)
-	{
+          {
+            case 0:
+                /* getopt_long took care of setting the flag. */ 
+                break;
 	    case 'Q':
 		really_quiet = TRUE;
 		/* FALL THROUGH */
@@ -258,11 +381,7 @@ main (argc, argv)
 		logoff = TRUE;
 		break;
 	    case 'v':
-		(void) fputs (version_string, stdout);
-		(void) sprintf (tmp, "Patch Level: %d\n", PATCHLEVEL);
-		(void) fputs (tmp, stdout);
-		(void) fputs ("\nCopyright (c) 1993-1994 Brian Berliner\nCopyright (c) 1993-1994 david d `zoo' zuhn\nCopyright (c) 1992, Brian Berliner and Jeff Polk\nCopyright (c) 1989-1992, Brian Berliner\n\nCVS may be copied only under the terms of the GNU General Public License,\na copy of which can be found with the CVS distribution kit.\n", stdout);
-		exit (0);
+                version_flag = TRUE;
 		break;
 	    case 'b':
 		Rcsbin = optarg;
@@ -283,16 +402,42 @@ main (argc, argv)
 		use_cvsrc = FALSE;
 		break;
 	    case 'z':
+#ifdef CLIENT_SUPPORT
 		gzip_level = atoi (optarg);
 		if (gzip_level <= 0 || gzip_level > 9)
 		  error (1, 0,
 			 "gzip compression level must be between 1 and 9");
+#endif
+		/* If no CLIENT_SUPPORT, we just silently ignore the gzip
+		   level, so that users can have it in their .cvsrc and not
+		   cause any trouble.  */
+		break;
+	    case 's':
+		variable_set (optarg);
 		break;
 	    case '?':
 	    default:
-		usage (usg);
+                usage (usg);
 	}
     }
+
+    if (version_flag == TRUE)
+    {
+        (void) fputs (version_string, stdout);
+        (void) fputs (config_string, stdout);
+        (void) fputs ("\n", stdout);
+        (void) fputs ("Copyright (c) 1993-1994 Brian Berliner\n", stdout);
+        (void) fputs ("Copyright (c) 1993-1994 david d `zoo' zuhn\n", stdout);
+        (void) fputs ("Copyright (c) 1992, Brian Berliner and Jeff Polk\n", stdout);
+        (void) fputs ("Copyright (c) 1989-1992, Brian Berliner\n", stdout);
+        (void) fputs ("\n", stdout);
+        (void) fputs ("CVS may be copied only under the terms of the GNU General Public License,\n", stdout);
+        (void) fputs ("a copy of which can be found with the CVS distribution kit.\n", stdout);
+        exit (0);
+    }
+    else if (help_commands)
+	usage (cmd_usage);
+
     argc -= optind;
     argv += optind;
     if (argc < 1)
@@ -325,7 +470,7 @@ main (argc, argv)
 	{
 	    printf ("E Fatal error, aborting.\n\
 error %s getpeername or getsockname failed\n", strerror (errno));
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	}
 
 	status = krb_recvauth (KOPT_DO_MUTUAL, STDIN_FILENO, &ticket, "rcmd",
@@ -335,7 +480,7 @@ error %s getpeername or getsockname failed\n", strerror (errno));
 	{
 	    printf ("E Fatal error, aborting.\n\
 error 0 kerberos: %s\n", krb_get_err_text(status));
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	}
 
 	/* Get the local name.  */
@@ -344,7 +489,7 @@ error 0 kerberos: %s\n", krb_get_err_text(status));
 	{
 	    printf ("E Fatal error, aborting.\n\
 error 0 kerberos: can't get local name: %s\n", krb_get_err_text(status));
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	}
 
 	pw = getpwnam (user);
@@ -352,7 +497,7 @@ error 0 kerberos: can't get local name: %s\n", krb_get_err_text(status));
 	{
 	    printf ("E Fatal error, aborting.\n\
 error 0 %s: no such user\n", user);
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	}
 
 	initgroups (pw->pw_name, pw->pw_gid);
@@ -383,7 +528,21 @@ error 0 %s: no such user\n", user);
     }
 #endif /* HAVE_KERBEROS */
 
-#ifdef CVSADM_ROOT
+
+#if defined(AUTH_SERVER_SUPPORT) && defined(SERVER_SUPPORT)
+    if (strcmp (argv[0], "pserver") == 0)
+    {
+      /* Gets username and password from client, authenticates, then
+         switches to run as that user and sends an ACK back to the
+         client. */
+      authenticate_connection ();
+      
+      /* Pretend we were invoked as a plain server.  */
+      argv[0] = "server";
+    }
+#endif /* AUTH_SERVER_SUPPORT && SERVER_SUPPORT */
+
+
     /*
      * See if we are able to find a 'better' value for CVSroot in the
      * CVSADM_ROOT directory.
@@ -398,45 +557,47 @@ error 0 %s: no such user\n", user);
 #endif /* No SERVER_SUPPORT */
     if (CVSADM_Root != NULL)
     {
-        if (CVSroot == NULL)
+        if (CVSroot == NULL || !cvs_update_env)
         {
 	    CVSroot = CVSADM_Root;
 	    cvs_update_env = 1;	/* need to update environment */
         }
 #ifdef CLIENT_SUPPORT
         else if (!getenv ("CVS_IGNORE_REMOTE_ROOT"))
-#else
+#else /* ! CLIENT_SUPPORT */
         else
-#endif
+#endif /* CLIENT_SUPPORT */
         {
             /*
 	     * Now for the hard part, compare the two directories. If they
 	     * are not identical, then abort this command.
 	     */
-            if ((strcmp (CVSroot, CVSADM_Root) != 0) &&
+            if ((fncmp (CVSroot, CVSADM_Root) != 0) &&
 		!same_directories(CVSroot, CVSADM_Root))
 	    {
-	        error (0, 0, "%s value for CVS Root found in %s",
-		       CVSADM_Root, CVSADM_ROOT);
-		if (cvs_update_env)
-		{
-		    error (0, 0, "does not match command line -d %s setting",
-			   CVSroot);
-		    error (1, 0,
-			   "you may wish to try the cvs command again without the -d option ");
-                }
-		else
-		{
-		    error (0, 0,
-			   "does not match CVSROOT environment value of %s",
-			   CVSroot);
-		    error (1, 0,
-			   "you may wish to unsetenv CVSROOT and try again");
-                }
+              error (0, 0, "%s value for CVS Root found in %s",
+                     CVSADM_Root, CVSADM_ROOT);
+              error (0, 0, "does not match command line -d %s setting",
+                     CVSroot);
+              error (1, 0,
+                      "you may wish to try the cvs command again without the -d option ");
 	    }
         }
     }
-#endif /* CVSADM_ROOT */
+
+    /* CVSroot may need fixing up, if an access-method was specified,
+     * but not a user.  Later code assumes that if CVSroot contains an
+     * access-method, then it also has a user.  We print a warning and
+     * die if we can't guarantee that.
+     */
+    if (CVSroot
+        && *CVSroot
+        && (CVSroot[0] == ':')
+        && (strchr (CVSroot, '@') == NULL))
+      {
+        error (1, 0,
+               "must also give a username if specifying access method");
+      }
 
     /*
      * Specifying just the '-H' flag to the sub-command causes a Usage
@@ -463,23 +624,29 @@ error 0 %s: no such user\n", user);
 		error (1, 0, "You don't have a %s environment variable",
 		       CVSROOT_ENV);
 	    (void) sprintf (path, "%s/%s", CVSroot, CVSROOTADM);
-	    if (access (path, R_OK | X_OK))
+	    if (!isaccessible (path, R_OK | X_OK))
 	    {
 		save_errno = errno;
+		/* If this is "cvs init", the root need not exist yet.  */
+		if (strcmp (command_name, "init") != 0
 #ifdef CLIENT_SUPPORT
-		if (strchr (CVSroot, ':') == NULL)
-		{
+		    /* If we are a remote client, the root need not exist
+		       on the client machine (FIXME: we should also skip
+		       the check for CVSROOTADM_HISTORY being writable;
+		       it shouldn't matter if there is a read-only file
+		       which happens to have the same name on the client
+		       machine).  */
+		    && strchr (CVSroot, ':') == NULL)
 #endif
+		{
 		error (0, 0,
 		    "Sorry, you don't have sufficient access to %s", CVSroot);
 		error (1, save_errno, "%s", path);
-#ifdef CLIENT_SUPPORT
 		}
-#endif
 	    }
 	    (void) strcat (path, "/");
 	    (void) strcat (path, CVSROOTADM_HISTORY);
-	    if (isfile (path) && access (path, R_OK | W_OK))
+	    if (isfile (path) && !isaccessible (path, R_OK | W_OK))
 	    {
 		save_errno = errno;
 		error (0, 0,
@@ -556,17 +723,26 @@ error 0 %s: no such user\n", user);
 	command_name = cm->fullname;	/* Global pointer for later use */
 
 	/* make sure we clean up on error */
+#ifdef SIGHUP
 	(void) SIG_register (SIGHUP, main_cleanup);
-	(void) SIG_register (SIGINT, main_cleanup);
-	(void) SIG_register (SIGQUIT, main_cleanup);
-	(void) SIG_register (SIGPIPE, main_cleanup);
-	(void) SIG_register (SIGTERM, main_cleanup);
-
 	(void) SIG_register (SIGHUP, Lock_Cleanup);
+#endif
+#ifdef SIGINT
+	(void) SIG_register (SIGINT, main_cleanup);
 	(void) SIG_register (SIGINT, Lock_Cleanup);
+#endif
+#ifdef SIGQUIT
+	(void) SIG_register (SIGQUIT, main_cleanup);
 	(void) SIG_register (SIGQUIT, Lock_Cleanup);
+#endif
+#ifdef SIGPIPE
+	(void) SIG_register (SIGPIPE, main_cleanup);
 	(void) SIG_register (SIGPIPE, Lock_Cleanup);
+#endif
+#ifdef SIGTERM
+	(void) SIG_register (SIGTERM, main_cleanup);
 	(void) SIG_register (SIGTERM, Lock_Cleanup);
+#endif
 
 	gethostname(hostname, sizeof (hostname));
 
@@ -580,7 +756,7 @@ error 0 %s: no such user\n", user);
 #endif
 
 	if (use_cvsrc)
-	  read_cvsrc(&argc, &argv);
+	  read_cvsrc (&argc, &argv, command_name);
 
 #ifdef CLIENT_SUPPORT
 	/* If cvsroot contains a colon, try to do it via the protocol.  */
@@ -596,14 +772,10 @@ error 0 %s: no such user\n", user);
 
 #endif /* No CLIENT_SUPPORT */
     }
-    /*
-     * If the command's error count is modulo 256, we need to change it
-     * so that we don't overflow the 8-bits we get to report exit status
-     */
-    if (err && (err % 256) == 0)
-	err = 1;
     Lock_Cleanup ();
-    return (err);
+    if (err)
+	return (EXIT_FAILURE);
+    return 0;
 }
 
 char *
@@ -638,5 +810,5 @@ usage (cpp)
     (void) fprintf (stderr, *cpp++, program_name, command_name);
     for (; *cpp; cpp++)
 	(void) fprintf (stderr, *cpp);
-    exit (1);
+    exit (EXIT_FAILURE);
 }
