@@ -17,14 +17,7 @@
 
 #include "cvs.h"
 
-#ifndef lint
-static const char rcsid[] = "$CVSid: @(#)remove.c 1.39 94/10/07 $";
-USE(rcsid)
-#endif
-
-static int remove_fileproc PROTO((char *file, char *update_dir,
-			    char *repository, List *entries,
-			    List *srcfiles));
+static int remove_fileproc PROTO((struct file_info *finfo));
 static Dtype remove_dirproc PROTO((char *dir, char *repos, char *update_dir));
 
 static int force;
@@ -74,21 +67,25 @@ cvsremove (argc, argv)
     argc -= optind;
     argv += optind;
 
+    wrap_setup ();
+
+#ifdef CLIENT_SUPPORT
     if (client_active) {
 	start_server ();
 	ign_setup ();
 	if (local)
 	    send_arg("-l");
+	send_file_names (argc, argv, SEND_EXPAND_WILD);
 	send_files (argc, argv, local, 0);
-	if (fprintf (to_server, "remove\n") < 0)
-	    error (1, errno, "writing to server");
+	send_to_server ("remove\012", 0);
         return get_responses_and_close ();
     }
+#endif
 
     /* start the recursion processor */
-    err = start_recursion (remove_fileproc, (int (*) ()) NULL, remove_dirproc,
-			   (int (*) ()) NULL, argc, argv, local,
-			   W_LOCAL, 0, 1, (char *) NULL, 1, 0);
+    err = start_recursion (remove_fileproc, (FILESDONEPROC) NULL,
+                           remove_dirproc, (DIRLEAVEPROC) NULL, argc, argv,
+                           local, W_LOCAL, 0, 1, (char *) NULL, 1, 0);
 
     if (removed_files)
 	error (0, 0, "use '%s commit' to remove %s permanently", program_name,
@@ -97,9 +94,9 @@ cvsremove (argc, argv)
     if (existing_files)
 	error (0, 0,
 	       ((existing_files == 1) ?
-		"%d file exists; use `%s' to remove it first" :
-		"%d files exist; use `%s' to remove them first"),
-	       existing_files, RM);
+		"%d file exists; remove it first" :
+		"%d files exist; remove them first"),
+	       existing_files);
 
     return (err);
 }
@@ -109,73 +106,78 @@ cvsremove (argc, argv)
  */
 /* ARGSUSED */
 static int
-remove_fileproc (file, update_dir, repository, entries, srcfiles)
-    char *file;
-    char *update_dir;
-    char *repository;
-    List *entries;
-    List *srcfiles;
+remove_fileproc (finfo)
+    struct file_info *finfo;
 {
     char fname[PATH_MAX];
     Vers_TS *vers;
 
-    /*
-     * If unlinking the file works, good.  If not, the "unremoved"
-     * error will indicate problems.
-     */
     if (force)
-	(void) unlink (file);
+    {
+	if (!noexec)
+	{
+	    if (unlink (finfo->file) < 0 && ! existence_error (errno))
+	    {
+		error (0, errno, "unable to remove %s", finfo->fullname);
+	    }
+	}
+	/* else FIXME should probably act as if the file doesn't exist
+	   in doing the following checks.  */
+    }
 
-    vers = Version_TS (repository, (char *) NULL, (char *) NULL, (char *) NULL,
-		       file, 0, 0, entries, srcfiles);
+    vers = Version_TS (finfo->repository, (char *) NULL, (char *) NULL, (char *) NULL,
+		       finfo->file, 0, 0, finfo->entries, finfo->rcs);
 
     if (vers->ts_user != NULL)
     {
 	existing_files++;
 	if (!quiet)
-	    error (0, 0, "file `%s' still in working directory", file);
+	    error (0, 0, "file `%s' still in working directory",
+		   finfo->fullname);
     }
     else if (vers->vn_user == NULL)
     {
 	if (!quiet)
-	    error (0, 0, "nothing known about `%s'", file);
+	    error (0, 0, "nothing known about `%s'", finfo->fullname);
     }
     else if (vers->vn_user[0] == '0' && vers->vn_user[1] == '\0')
     {
 	/*
 	 * It's a file that has been added, but not commited yet. So,
-	 * remove the ,p and ,t file for it and scratch it from the
-	 * entries file.
-	 */
-	Scratch_Entry (entries, file);
-	(void) sprintf (fname, "%s/%s%s", CVSADM, file, CVSEXT_OPT);
-	(void) unlink_file (fname);
-	(void) sprintf (fname, "%s/%s%s", CVSADM, file, CVSEXT_LOG);
+	 * remove the ,t file for it and scratch it from the
+	 * entries file.  */
+	Scratch_Entry (finfo->entries, finfo->file);
+	(void) sprintf (fname, "%s/%s%s", CVSADM, finfo->file, CVSEXT_LOG);
 	(void) unlink_file (fname);
 	if (!quiet)
-	    error (0, 0, "removed `%s'", file);
+	    error (0, 0, "removed `%s'", finfo->fullname);
 
+#ifdef SERVER_SUPPORT
 	if (server_active)
-	    server_checked_in (file, update_dir, repository);
+	    server_checked_in (finfo->file, finfo->update_dir, finfo->repository);
+#endif
     }
     else if (vers->vn_user[0] == '-')
     {
 	if (!quiet)
-	    error (0, 0, "file `%s' already scheduled for removal", file);
+	    error (0, 0, "file `%s' already scheduled for removal",
+		   finfo->fullname);
     }
     else
     {
 	/* Re-register it with a negative version number.  */
 	(void) strcpy (fname, "-");
 	(void) strcat (fname, vers->vn_user);
-	Register (entries, file, fname, vers->ts_rcs, vers->options,
+	Register (finfo->entries, finfo->file, fname, vers->ts_rcs, vers->options,
 		  vers->tag, vers->date, vers->ts_conflict);
 	if (!quiet)
-	    error (0, 0, "scheduling `%s' for removal", file);
+	    error (0, 0, "scheduling `%s' for removal", finfo->fullname);
 	removed_files++;
 
+#ifdef SERVER_SUPPORT
 	if (server_active)
-	    server_checked_in (file, update_dir, repository);
+	    server_checked_in (finfo->file, finfo->update_dir, finfo->repository);
+#endif
     }
 
     freevers_ts (&vers);
